// Gigster
const {Module} = require('@gigster/gig-sdk');
const {Loopback, Node} = require('@gigster/gig-frameworks');
const {
  JsonService,
  LogService,
} = require('@gigster/gig-services');

/**
 * LoopbackFirebaseAuthenticationModule adds a firebase token to
 * the LoopbackAuthenticationModule so that when a user logins in,
 * they are able to then authenticate with firebase with this token
 * from the clientside.
 * @class
 */
class LoopbackFirebaseAuthenticationModule extends Module {

  // --------------------------------------------------
  // Initialize
  // --------------------------------------------------
  constructor(options) {
    super(options);
    this.loopback = new Loopback({module: this});
    this.node = new Node({module: this});
  }

  // --------------------------------------------------
  // Config
  // --------------------------------------------------

  /**
   * Provides the contribution to the CLI for the project.
   */
  get contributions() {
    return require('../config/contributions.json'); // eslint-disable-line
  }

  // --------------------------------------------------
  // Properties
  // --------------------------------------------------

  get accessTokenJS() {
    return this.loopback.getServerPath(
      'models',
      'accessToken',
      'accessToken.js',
    );
  }

  get firebaseJSON() {
    return this.loopback.getServerPath(
      'models',
      'firebase',
      'firebase.json',
    );
  }

  // --------------------------------------------------
  // Generate
  // --------------------------------------------------

  /**
   * Generates the module.
   * @method
   */
  generate() {
    LogService.verbose('Generating Loopback Firebase Authentication module.');
    this.node.mergeConfigPackageJSON();
    this.updateAccessTokenJS();
    this.updateAccessTokenJSON();
  }

  updateAccessTokenJS() {
    LogService.verbose('Updating accessToken.js');
    this.loopback.addIncludeToModel({
      path: this.accessTokenJS,
      model: 'accessToken',
      include: 'firebase',
    });
    this.copyFiles();
    this.copyModule();
  }

  updateAccessTokenJSON() {
    LogService.verbose('Updating firebase.json');

    const firebaseJSON = JsonService.read(this.firebaseJSON);
    firebaseJSON.plural = 'firebase';
    firebaseJSON.mixins = {
      SetupRemoteMethods: {
        disableAllExcept: ['refreshToken'],
      },
    };
    JsonService.overwrite(this.firebaseJSON, firebaseJSON);
  }

}

module.exports = LoopbackFirebaseAuthenticationModule;
